
# q-expansion of newform 1200.3.l.l, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1200
weight = 3
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[751, [1, 0]], [901, [1, 0]], [401, [-1, 0]], [577, [1, 0]]]
aps_data = [[0, 0], [0, 3], [0, 0], [0, 0], [0, 0], [0, 0], [0, 14], [-22, 0], [0, -34], [0, 0], [-2, 0], [0, 0], [0, 0], [0, 0], [0, -14], [0, -86], [0, 0], [-118, 0], [0, 0], [0, 0], [0, 0], [98, 0], [0, -154], [0, 0], [0, 0], [0, 0], [0, 0], [0, 106], [22, 0], [0, -206], [0, 0], [0, 0], [0, -226], [-262, 0], [0, 0], [238, 0], [0, 0], [0, 0], [0, -254], [0, 154], [0, 0], [122, 0], [0, 0], [0, 0], [0, 374], [-142, 0], [-362, 0], [0, 0], [0, -134], [-218, 0], [0, 34], [0, 0], [-478, 0], [0, 0], [0, -466], [0, 446], [0, 0], [-482, 0], [0, 0], [0, 0], [0, 0], [0, 394], [0, 0], [0, 0], [0, 0], [0, 134], [-122, 0], [0, 0], [0, 586], [-458, 0], [0, 274], [0, 0], [0, 0], [0, 0], [-742, 0], [0, 686], [0, 0], [0, 0], [0, 0], [142, 0], [0, 0], [602, 0], [0, 0], [0, 0], [-622, 0], [0, 566], [0, 0], [0, 0], [0, 0], [0, 0], [0, 346], [0, 0], [0, 0], [0, 0], [938, 0], [0, -994], [0, 0], [0, 0], [0, 0], [-1078, 0], [0, 0], [0, 614], [0, -154], [0, 0], [358, 0], [0, 0], [0, -854], [0, -1166], [0, 0], [242, 0], [0, 0], [0, 0], [0, -1186], [698, 0], [238, 0], [0, 0], [0, 0], [0, 706], [0, 1114], [0, 0], [-838, 0], [0, 0], [0, 374], [0, 86], [-1322, 0], [0, 0], [742, 0], [0, 0], [0, 0], [0, 0], [-1462, 0], [0, -514], [1438, 0], [0, 0], [0, 0], [-578, 0], [0, -1526], [0, 0], [0, -826], [0, 0], [-1082, 0], [0, 0], [0, 0], [0, -374], [502, 0], [0, 0], [0, 0], [0, -1666], [218, 0], [0, -274], [0, 0], [0, 0], [0, 0], [0, -1694], [0, 0], [0, 0], [1298, 0], [0, 0], [0, 0], [0, 0], [0, -1574], [0, 1474], [0, 0], [0, 0], [0, 1934], [0, -1954], [958, 0], [0, 0], [1822, 0], [0, -1046], [0, 0], [1802, 0], [0, 0], [0, 0], [2018, 0], [0, 0], [-602, 0], [0, 0], [0, 0], [22, 0], [0, 0], [0, 0], [0, 0], [0, 1694], [0, 2126], [0, 0], [0, 0], [0, 0], [1582, 0], [0, 0], [0, 0], [0, 2006], [-2282, 0], [0, 0], [0, -1094], [0, 34], [1442, 0], [0, 0], [0, 1454], [0, -1474], [0, 0], [2398, 0], [0, 0], [1342, 0], [0, 0], [0, -826], [-2302, 0], [0, -2554], [0, 0], [-2042, 0], [0, 0], [0, 0], [0, 0], [0, 2506], [0, 0], [-1198, 0], [0, 0], [0, 0], [0, -734], [0, -326], [602, 0], [1298, 0], [0, 0], [0, 0], [0, 2266], [-2618, 0], [0, 514], [0, 0], [0, 0], [0, 0], [0, 0], [2858, 0], [1918, 0], [0, 0], [0, 0], [0, 946], [862, 0], [0, 2794], [0, 0], [0, 0], [0, -2074], [-1562, 0], [0, 0], [2902, 0], [0, 2674], [0, 0], [0, 0], [0, 0], [2618, 0], [0, 1166], [0, 0], [0, 0], [0, -3134], [-2258, 0], [0, 154], [0, 0], [-2758, 0], [0, 0], [0, -2506], [0, 0], [0, 0], [0, -3014], [2662, 0], [0, 0], [0, 2414], [-1462, 0], [0, 0], [0, 0], [0, 0], [0, -3446], [3242, 0], [0, 0], [0, 0], [578, 0], [0, 0], [0, 0], [0, 1546], [2422, 0], [-238, 0], [0, 0], [0, 1646], [3598, 0], [0, 226], [-2278, 0], [0, 0], [0, 0], [0, 0], [0, -2026], [-3502, 0], [0, 0], [0, 0], [0, -2534], [0, 3394], [0, 0], [0, 0], [0, 0], [-3842, 0], [0, -2966], [0, 0], [0, 0], [0, 0], [0, -3226], [-862, 0]]
